const fs = require('fs');
const path = require('path');
const { app, autoUpdater, dialog } = require('electron');

const pkg = require('./package.json');

module.exports.isInstalled = function () {
  // check if packaged
  if (!app.isPackaged) {
    return false;
  }

  // check platform and whether app is in correct location
  switch (process.platform) {
    case 'darwin':
      // on macOS the app is installed if it has been moved to the applications folder
      return app.isInApplicationsFolder();
    case 'win32':
      // on Windows the app is installed if the parent directory contains the update program
      return fs.existsSync(path.resolve(path.dirname(process.execPath), '..', 'Update.exe'));
    default:
      return false;
  }
};

module.exports.autoUpdate = function (server = '', notify = true, interval = 300, reporter = () => {}) {
  // determine format
  const format = global.process.platform === 'win32' ? 'nupkg' : 'zip';

  // set feed url
  autoUpdater.setFeedURL({
    url: `${server}/${process.platform}-${process.arch}-${format}/${app.getVersion()}`,
    headers: {
      'User-Agent': `comet-electron (${pkg.version}, ${process.platform}, ${process.arch})`,
    },
  });

  // handle errors
  autoUpdater.on('error', (err) => {
    reporter(err);
    console.log('comet: auto: updater error:', err);
  });

  // log event
  autoUpdater.on('checking-for-update', () => {
    console.log('comet: auto: checking for update');
  });

  // log event
  autoUpdater.on('update-available', () => {
    console.log('comet: auto: update available');
  });

  // log event
  autoUpdater.on('update-not-available', () => {
    console.log('comet: auto: no update available');
  });

  // handle update
  autoUpdater.on('update-downloaded', async (event, notes, version, published, url) => {
    console.log('comet: auto: update downloaded:', version, notes, published, url);

    // check flag
    if (!notify) {
      return;
    }

    // notify user
    const { response } = await dialog.showMessageBox({
      type: 'info',
      title: 'Application Update',
      message: `Version ${version} has been downloaded.`,
      detail: 'Restart the application to apply the update.',
      buttons: ['Restart', 'Skip'],
      defaultId: 0,
    });

    // check response
    if (response === 0) {
      autoUpdater.quitAndInstall();
    }

    // notify only once
    notify = false;
  });

  // perform initial check
  autoUpdater.checkForUpdates();

  // perform periodic checks
  setInterval(() => {
    autoUpdater.checkForUpdates();
  }, interval * 1000);
};
