const electron = require('electron');

const { isInstalled, autoUpdate } = require('./auto');
const { sleep } = require('./utils');

jest.mock('electron', () => {
  return {
    app: {
      getVersion() {
        return '1.2.3';
      },
      isPackaged: false,
      isInApplicationsFolder: jest.fn(),
    },
    autoUpdater: {
      on: jest.fn(),
      setFeedURL: jest.fn(),
      checkForUpdates: jest.fn(),
      quitAndInstall: jest.fn(),
    },
    dialog: {
      showMessageBox: jest.fn(),
    },
  };
});

jest.mock('./package.json', () => {
  return {
    version: '1.2.3',
  };
});

jest.mock('process');

test('isInstalled', async () => {
  expect(isInstalled()).toBe(false);

  electron.app.isPackaged = true;

  Object.defineProperty(global.process, 'platform', { value: 'darwin' });
  electron.app.isInApplicationsFolder.mockImplementation(() => {
    return true;
  });

  expect(isInstalled()).toBe(true);

  Object.defineProperty(global.process, 'platform', { value: 'win32' });

  expect(isInstalled()).toBe(false);

  Object.defineProperty(global.process, 'platform', { value: 'linux' });

  expect(isInstalled()).toBe(false);
});

test('autoUpdate', async () => {
  let events = {};
  electron.autoUpdater.on.mockImplementation((event, cb) => (events[event] = cb));
  Object.defineProperty(global.process, 'platform', { value: 'darwin' });

  /* Initialization */

  let reporter = jest.fn();
  autoUpdate('https://update.example.org', true, 5, reporter);
  await sleep(100);

  expect(electron.autoUpdater.setFeedURL).toHaveBeenLastCalledWith({
    url: `https://update.example.org/darwin-${process.arch}-zip/1.2.3`,
    headers: {
      'User-Agent': `comet-electron (1.2.3, darwin, ${process.arch})`,
    },
  });

  expect(Object.keys(events)).toEqual([
    'error',
    'checking-for-update',
    'update-available',
    'update-not-available',
    'update-downloaded',
  ]);

  expect(electron.autoUpdater.checkForUpdates).toBeCalled();

  /* Initialization (windows) */

  Object.defineProperty(global.process, 'platform', { value: 'win32' });

  reporter = jest.fn();
  autoUpdate('https://update.example.org', true, 5, reporter);
  await sleep(100);

  expect(electron.autoUpdater.setFeedURL).toHaveBeenLastCalledWith({
    url: `https://update.example.org/win32-${process.arch}-nupkg/1.2.3`,
    headers: {
      'User-Agent': `comet-electron (1.2.3, win32, ${process.arch})`,
    },
  });

  Object.defineProperty(global.process, 'platform', { value: 'darwin' });

  /* Update Downloaded */

  electron.dialog.showMessageBox.mockResolvedValue({ response: 0 });
  await events['update-downloaded'](null, 'notes', '3.2.1', new Date(), 'https://download.example.org/3.2.1');
  expect(electron.dialog.showMessageBox).toBeCalledWith({
    type: 'info',
    title: 'Application Update',
    message: 'Version 3.2.1 has been downloaded.',
    detail: 'Restart the application to apply the update.',
    buttons: ['Restart', 'Skip'],
    defaultId: 0,
  });

  expect(electron.autoUpdater.quitAndInstall).toBeCalled();

  /* Errors */

  reporter = jest.fn();
  autoUpdate('https://update.example.org', true, 5, reporter);
  await sleep(100);

  events['error']('electron error');
  expect(reporter).toBeCalledWith('electron error');
});
