const { app } = require('electron');

const { retry, sleep } = require('./utils');
const { lookup } = require('./config');
const { isInstalled, autoUpdate } = require('./auto');
const { manualUpdate } = require('./manual');

/**
 * Update expects a TXT record at the specified domain with the following format:
 *
 *  {
 *    server: 'https://comet.sternenbauer.com/update/shiftr-io-desktop',
 *    site: 'https://next.shiftr.io/desktop#downloads',
 *    delay: 5,
 *    interval: 300,
 *    notify: true
 *  }
 *
 *  @param domain The config domain.
 *  @param reporter The error reporter.
 */
module.exports = async function update(domain = '', reporter = () => {}) {
  // handle windows squirrel events
  if (process.platform === 'win32' && require('electron-squirrel-startup')) {
    app.quit();
    return;
  }

  // ensure app is ready
  if (!app.isReady()) {
    app.on('ready', () => {
      update(...arguments);
    });
    return;
  }

  // initialize
  retry(1000, async () => {
    // get config
    let config;
    try {
      config = await lookup(domain);
    } catch (err) {
      reporter(err);
      console.log('comet: failed to lookup config:', err);
      return false;
    }

    // log config
    console.log('comet: got config:', config);

    // await configured delay
    await sleep(config.delay * 1000);

    // try automatic updates if installed
    if (isInstalled()) {
      try {
        autoUpdate(config.server, config.notify, config.interval, reporter);
        return true;
      } catch (err) {
        reporter(err);
        console.log('comet: unable to use auto update:', err);
      }
    }

    // otherwise fallback to manual update
    manualUpdate(config.server, config.site, config.interval, reporter);

    return true;
  }).then();
};
